searchData={"items":[{"type":"module","title":"OpenAI.Responses","doc":"Client for OpenAI Responses API.\n\nThis module provides a simple interface for creating AI responses with support for:\n- Text and structured output generation\n- Streaming responses with Server-Sent Events (SSE)\n- Automatic cost calculation for all API calls\n- JSON Schema-based structured outputs","ref":"OpenAI.Responses.html"},{"type":"module","title":"Available Functions - OpenAI.Responses","doc":"- `create/1` and `create/2` - Create AI responses (synchronous or streaming)\n- `create!/1` and `create!/2` - Same as create but raises on error\n- `run/2` and `run!/2` - Run conversations with automatic function calling\n- `call_functions/2` - Execute function calls and format results for the API\n- `stream/1` - Stream responses as an Enumerable\n- `list_models/0` and `list_models/1` - List available OpenAI models\n- `request/1` - Low-level API request function","ref":"OpenAI.Responses.html#module-available-functions"},{"type":"module","title":"Configuration - OpenAI.Responses","doc":"Set your OpenAI API key via:\n- Environment variable: `OPENAI_API_KEY`\n- Application config: `config :openai_responses, :openai_api_key, \"your-key\"`","ref":"OpenAI.Responses.html#module-configuration"},{"type":"module","title":"Examples - OpenAI.Responses","doc":"See the [tutorial](tutorial.livemd) for comprehensive examples and usage patterns.","ref":"OpenAI.Responses.html#module-examples"},{"type":"function","title":"OpenAI.Responses.call_functions/2","doc":"Execute function calls from a response and format the results for the API.\n\nTakes the function_calls from a response and a map/keyword list of functions,\nexecutes each function with its arguments, and returns the formatted results\nready to be used as input for the next API call.","ref":"OpenAI.Responses.html#call_functions/2"},{"type":"function","title":"Parameters - OpenAI.Responses.call_functions/2","doc":"- `function_calls` - The function_calls array from a Response struct\n- `functions` - A map or keyword list where:\n  - Keys are function names (as atoms or strings)\n  - Values are functions that accept the parsed arguments and return the result","ref":"OpenAI.Responses.html#call_functions/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.call_functions/2","doc":"Returns a list of formatted function outputs suitable for use as input to `create/2`.\n\n**Important**: Function return values must be JSON-encodable. This means they should\nonly contain basic types (strings, numbers, booleans, nil), lists, and maps. Tuples,\natoms (except `true`, `false`, and `nil`), and other Elixir-specific types are not\nsupported by default unless they implement the `Jason.Encoder` protocol.","ref":"OpenAI.Responses.html#call_functions/2-returns"},{"type":"function","title":"Examples - OpenAI.Responses.call_functions/2","doc":"# Get a response with function calls\n    {:ok, response} = Responses.create(\n      input: \"What's the weather in Paris and what time is it?\",\n      tools: [weather_tool, time_tool]\n    )\n\n    # Define the actual function implementations\n    functions = %{\n      \"get_weather\" => fn %{\"location\" => location} ->\n        # Returns a map (JSON-encodable)\n        %{temperature: 22, unit: \"C\", location: location}\n      end,\n      \"get_time\" => fn %{} ->\n        # Returns a string (JSON-encodable)\n        DateTime.utc_now() |> to_string()\n      end\n    }\n\n    # Execute the functions and get formatted output\n    outputs = Responses.call_functions(response.function_calls, functions)\n\n    # Continue the conversation with the function results\n    {:ok, final_response} = Responses.create(response, input: outputs)","ref":"OpenAI.Responses.html#call_functions/2-examples"},{"type":"function","title":"Error Handling - OpenAI.Responses.call_functions/2","doc":"If a function is not found or raises an error, the output will contain\nan error message instead of the function result.","ref":"OpenAI.Responses.html#call_functions/2-error-handling"},{"type":"function","title":"OpenAI.Responses.create/1","doc":"Create a new response.\n\nWhen the argument is a string, it is used as the input text.\nOtherwise, the argument is expected to be a keyword list or map of options that OpenAI expects,\nsuch as `input`, `model`, `temperature`, `max_tokens`, etc.","ref":"OpenAI.Responses.html#create/1"},{"type":"function","title":"Model Behavior with previous_response_id - OpenAI.Responses.create/1","doc":"The OpenAI API always requires a model parameter, even when using `previous_response_id`.\n\nWhen using `create/1` with manual `previous_response_id`:\n- If no model is specified, the default model is used\n- The model from the previous response is NOT automatically inherited\n\nWhen using `create/2` with a Response object:\n- The model from the previous response IS automatically inherited\n- You can override it by explicitly specifying a different model\n\n    # Manual previous_response_id - uses default model if not specified\n    Responses.create(input: \"Hello\", previous_response_id: \"resp_123\")\n\n    # Manual previous_response_id - with explicit model\n    Responses.create(input: \"Hello\", previous_response_id: \"resp_123\", model: \"gpt-4.1\")\n\n    # Using create/2 - automatically inherits model from previous response\n    Responses.create(previous_response, input: \"Hello\")","ref":"OpenAI.Responses.html#create/1-model-behavior-with-previous_response_id"},{"type":"function","title":"Examples - OpenAI.Responses.create/1","doc":"# Using a keyword list\n    Responses.create(input: \"Hello\", model: \"gpt-4.1\", temperature: 0.7)\n\n    # Using a map\n    Responses.create(%{input: \"Hello\", model: \"gpt-4.1\", temperature: 0.7})\n\n    # String shorthand\n    Responses.create(\"Hello\")","ref":"OpenAI.Responses.html#create/1-examples"},{"type":"function","title":"Streaming - OpenAI.Responses.create/1","doc":"Pass a `stream:` option with a callback function to stream the response.\nThe callback receives results wrapped in `{:ok, chunk}` or `{:error, reason}` tuples:\n\n    Responses.create(\n      input: \"Write a story\",\n      stream: fn\n        {:ok, %{event: \"response.output_text.delta\", data: %{\"delta\" => text}}} ->\n          IO.write(text)\n          :ok\n        {:error, reason} ->\n          IO.puts(\"Stream error: #{inspect(reason)}\")\n          :ok  # Continue despite errors\n        _ ->\n          :ok\n      end\n    )\n\nThe callback should return `:ok` to continue or `{:error, reason}` to stop the stream.\n\nFor simpler text streaming, use the `delta/1` helper:\n\n    Responses.create(\n      input: \"Write a story\",\n      stream: Responses.Stream.delta(&IO.write/1)\n    )\n\nIf no model is specified, the default model is used.","ref":"OpenAI.Responses.html#create/1-streaming"},{"type":"function","title":"OpenAI.Responses.create/2","doc":"Create a response based on a previous response.\n\nThis allows creating follow-up responses that maintain context from a previous response.\nThe previous response's ID is automatically included in the request.\n\nOptions can be provided as either a keyword list or a map.","ref":"OpenAI.Responses.html#create/2"},{"type":"function","title":"Examples - OpenAI.Responses.create/2","doc":"{:ok, first} = Responses.create(\"What is Elixir?\")\n    \n    # Using keyword list\n    {:ok, followup} = Responses.create(first, input: \"Tell me more about its concurrency model\")\n    \n    # Using map\n    {:ok, followup} = Responses.create(first, %{input: \"Tell me more about its concurrency model\"})","ref":"OpenAI.Responses.html#create/2-examples"},{"type":"function","title":"OpenAI.Responses.create!/1","doc":"Same as `create/1` but raises an error on failure.\n\nReturns the response directly instead of an {:ok, response} tuple.","ref":"OpenAI.Responses.html#create!/1"},{"type":"function","title":"Examples - OpenAI.Responses.create!/1","doc":"response = Responses.create!(\"Hello, world!\")\n    IO.puts(response.text)","ref":"OpenAI.Responses.html#create!/1-examples"},{"type":"function","title":"OpenAI.Responses.create!/2","doc":"Same as `create/2` but raises an error on failure.\n\nReturns the response directly instead of an {:ok, response} tuple.","ref":"OpenAI.Responses.html#create!/2"},{"type":"function","title":"Examples - OpenAI.Responses.create!/2","doc":"first = Responses.create!(\"What is Elixir?\")\n    followup = Responses.create!(first, input: \"Tell me more\")","ref":"OpenAI.Responses.html#create!/2-examples"},{"type":"function","title":"OpenAI.Responses.list_models/1","doc":"List available models.\n\nAccepts an optional `match` string to filter by model ID.","ref":"OpenAI.Responses.html#list_models/1"},{"type":"function","title":"OpenAI.Responses.request/1","doc":"Request a response from the OpenAI API.\n\nUsed as a building block by other functions in this module.\nAccepts that same arguments as `Req.request/1`.\nYou should provide `url`, `json`, `method`, and other options as needed.","ref":"OpenAI.Responses.html#request/1"},{"type":"function","title":"OpenAI.Responses.run/2","doc":"Run a conversation with automatic function calling.\n\nThis function automates the process of handling function calls by repeatedly calling the\nprovided functions and feeding their results back to the model until a final response\nwithout function calls is received.","ref":"OpenAI.Responses.html#run/2"},{"type":"function","title":"Parameters - OpenAI.Responses.run/2","doc":"- `options` - Keyword list or map of options to pass to `create/1`\n- `functions` - A map or keyword list where:\n  - Keys are function names (as atoms or strings)\n  - Values are functions that accept the parsed arguments and return the result","ref":"OpenAI.Responses.html#run/2-parameters"},{"type":"function","title":"Returns - OpenAI.Responses.run/2","doc":"Returns a list of all responses generated during the conversation, in chronological order.\nThe last response in the list will be the final answer without function calls.","ref":"OpenAI.Responses.html#run/2-returns"},{"type":"function","title":"Examples - OpenAI.Responses.run/2","doc":"# Define available functions\n    functions = %{\n      \"get_weather\" => fn %{\"location\" => location} ->\n        # Simulate weather API call\n        \"The weather in #{location} is 72°F and sunny\"\n      end,\n      \"get_time\" => fn %{} ->\n        DateTime.utc_now() |> to_string()\n      end\n    }\n\n    # Create function tools\n    weather_tool = Responses.Schema.build_function(\n      \"get_weather\",\n      \"Get current weather for a location\",\n      %{location: :string}\n    )\n\n    time_tool = Responses.Schema.build_function(\n      \"get_time\",\n      \"Get the current UTC time\",\n      %{}\n    )\n\n    # Run the conversation (with keyword list)\n    responses = Responses.run(\n      [input: \"What's the weather in Paris and what time is it?\",\n       tools: [weather_tool, time_tool]],\n      functions\n    )\n\n    # Or with map\n    responses = Responses.run(\n      %{input: \"What's the weather in Paris and what time is it?\",\n        tools: [weather_tool, time_tool]},\n      functions\n    )\n\n    # The last response contains the final answer\n    final_response = List.last(responses)\n    IO.puts(final_response.text)","ref":"OpenAI.Responses.html#run/2-examples"},{"type":"function","title":"OpenAI.Responses.run!/2","doc":"Same as `run/2` but raises an error on failure.\n\nReturns the list of responses directly instead of an {:ok, responses} tuple.","ref":"OpenAI.Responses.html#run!/2"},{"type":"function","title":"OpenAI.Responses.stream/1","doc":"Stream a response from the OpenAI API as an Enumerable.\n\nReturns a Stream that yields chunks with `event` and `data` keys.\n\nOptions can be provided as either a keyword list or a map.","ref":"OpenAI.Responses.html#stream/1"},{"type":"function","title":"Examples - OpenAI.Responses.stream/1","doc":"# Stream and handle all results\n    for result <- Responses.stream(\"Tell me a story\") do\n      case result do\n        {:ok, chunk} -> IO.inspect(chunk)\n        {:error, reason} -> IO.puts(\"Error: #{inspect(reason)}\")\n      end\n    end\n\n    # Process only text deltas, ignoring errors\n    Responses.stream(\"Write a poem\")\n    |> Stream.filter(fn\n      {:ok, %{event: \"response.output_text.delta\"}} -> true\n      _ -> false\n    end)\n    |> Stream.map(fn {:ok, chunk} -> chunk.data[\"delta\"] end)\n    |> Enum.each(&IO.write/1)\n\n    # Accumulate all text with error handling (using map)\n    result = Responses.stream(%{input: \"Explain quantum physics\"})\n             |> Enum.reduce(%{text: \"\", errors: []}, fn\n               {:ok, %{event: \"response.output_text.delta\", data: %{\"delta\" => delta}}}, acc ->\n                 %{acc | text: acc.text <> delta}\n               {:error, reason}, acc ->\n                 %{acc | errors: [reason | acc.errors]}\n               _, acc ->\n                 acc\n             end)","ref":"OpenAI.Responses.html#stream/1-examples"},{"type":"module","title":"OpenAI.Responses.Pricing","doc":"Pricing information for OpenAI models.\n\nPrices are in USD per million tokens, stored as Decimal values for precision.\nSource: https://platform.openai.com/docs/pricing, last updated on 2025-06-03.","ref":"OpenAI.Responses.Pricing.html"},{"type":"function","title":"OpenAI.Responses.Pricing.get_pricing/1","doc":"Get pricing information for a specific model.\n\nReturns a map with :input, :cached_input, and :output prices per million tokens as Decimal values,\nor nil if the model is not found.","ref":"OpenAI.Responses.Pricing.html#get_pricing/1"},{"type":"function","title":"OpenAI.Responses.Pricing.list_models/0","doc":"List all available models with pricing.","ref":"OpenAI.Responses.Pricing.html#list_models/0"},{"type":"module","title":"OpenAI.Responses.Response","doc":"Represent a response from the OpenAI API.\n\nThe `Response` struct contains the following fields:\n- `text`: The extracted text from the response body.\n- `parsed`: The parsed response body.\n- `parse_error`: A map containing error messages if parsing failed.\n- `function_calls`: An array of extracted function calls from the response.\n- `body`: The raw response body.\n- `cost`: The calculated cost of the response in USD.\n\nAll of the functions in this module act like \"plugs\", meaning they take a response as input and return a modified response as output.\n\nThe `extract_text/1` function extracts the text from the response body, updating the `text` field of the response.\nThe `extract_json/1` function extracts the data from the response body if it is a structured response.\nThe `extract_function_calls/1` function extracts function calls from the response body.\nThe `calculate_cost/1` function calculates the cost of the response based on token usage and model pricing.","ref":"OpenAI.Responses.Response.html"},{"type":"function","title":"OpenAI.Responses.Response.calculate_cost/1","doc":"Calculate the cost of the response based on token usage and model pricing.\n\nReturns the response with the `cost` field updated with a map containing:\n- `input_cost`: Cost for input tokens in USD (as Decimal)\n- `output_cost`: Cost for output tokens in USD (as Decimal)\n- `total_cost`: Total cost in USD (as Decimal)\n- `cached_discount`: Amount saved from cached tokens in USD (as Decimal) (if applicable)\n\nIf pricing information is not available for the model, or usage information\nis missing, the cost field will be set to zero for all cost categories.","ref":"OpenAI.Responses.Response.html#calculate_cost/1"},{"type":"function","title":"OpenAI.Responses.Response.extract_function_calls/1","doc":"Extract function calls from the response body.\n\nFunction calls are extracted from the response.body[\"output\"] array and transformed\ninto a more convenient format with parsed arguments.","ref":"OpenAI.Responses.Response.html#extract_function_calls/1"},{"type":"function","title":"OpenAI.Responses.Response.extract_json/1","doc":"Extract the data from the response body if it is a structured response.\n\nAutomatically extracts the text from the response body if it is not already extracted.","ref":"OpenAI.Responses.Response.html#extract_json/1"},{"type":"function","title":"OpenAI.Responses.Response.extract_text/1","doc":"Extract the text from the response body.","ref":"OpenAI.Responses.Response.html#extract_text/1"},{"type":"module","title":"OpenAI.Responses.Schema","doc":"Helper module for defining structured output schemas and function calling tools.\n\nConverts simple Elixir syntax into JSON Schema format for structured outputs and function parameters.","ref":"OpenAI.Responses.Schema.html"},{"type":"module","title":"Examples - OpenAI.Responses.Schema","doc":"","ref":"OpenAI.Responses.Schema.html#module-examples"},{"type":"module","title":"Structured Output Schema - OpenAI.Responses.Schema","doc":"iex> Responses.Schema.build_output(%{\n    ...>   name: {:string, description: \"The name of the user\"},\n    ...>   username: {:string, description: \"The username of the user. Must start with @\", pattern: \"^@[a-zA-Z0-9_]+$\"},\n    ...>   email: {:string, description: \"The email of the user\", format: \"email\"}\n    ...> })\n    %{\n      \"name\" => \"data\",\n      \"type\" => \"json_schema\",\n      \"strict\" => true,\n      \"schema\" => %{\n        \"type\" => \"object\",\n        \"properties\" => %{\n          \"name\" => %{\n            \"type\" => \"string\",\n            \"description\" => \"The name of the user\"\n          },\n          \"username\" => %{\n            \"type\" => \"string\",\n            \"description\" => \"The username of the user. Must start with @\",\n            \"pattern\" => \"^@[a-zA-Z0-9_]+$\"\n          },\n          \"email\" => %{\n            \"type\" => \"string\",\n            \"description\" => \"The email of the user\",\n            \"format\" => \"email\"\n          }\n        },\n        \"additionalProperties\" => false,\n        \"required\" => [\"name\", \"username\", \"email\"]\n      }\n    }","ref":"OpenAI.Responses.Schema.html#module-structured-output-schema"},{"type":"module","title":"Function Calling Tool - OpenAI.Responses.Schema","doc":"iex> Responses.Schema.build_function(\"get_weather\", \"Get current temperature for a given location.\", %{\n    ...>   location: {:string, description: \"City and country e.g. Bogotá, Colombia\"}\n    ...> })\n    %{\n      \"name\" => \"get_weather\",\n      \"type\" => \"function\",\n      \"strict\" => true,\n      \"description\" => \"Get current temperature for a given location.\",\n      \"parameters\" => %{\n        \"type\" => \"object\",\n        \"properties\" => %{\n          \"location\" => %{\n            \"type\" => \"string\",\n            \"description\" => \"City and country e.g. Bogotá, Colombia\"\n          }\n        },\n        \"additionalProperties\" => false,\n        \"required\" => [\"location\"]\n      }\n    }","ref":"OpenAI.Responses.Schema.html#module-function-calling-tool"},{"type":"function","title":"OpenAI.Responses.Schema.build_function/3","doc":"Builds a function calling tool schema.","ref":"OpenAI.Responses.Schema.html#build_function/3"},{"type":"function","title":"Parameters - OpenAI.Responses.Schema.build_function/3","doc":"- `name` - The function name\n  - `description` - A description of what the function does\n  - `parameters` - A map or keyword list of parameter definitions (same format as `build_output/1`)","ref":"OpenAI.Responses.Schema.html#build_function/3-parameters"},{"type":"function","title":"Example - OpenAI.Responses.Schema.build_function/3","doc":"iex> build_function(\"get_weather\", \"Get weather for a location\", %{\n    ...>   location: {:string, description: \"City name\"},\n    ...>   units: {:string, enum: [\"celsius\", \"fahrenheit\"], description: \"Temperature units\"}\n    ...> })","ref":"OpenAI.Responses.Schema.html#build_function/3-example"},{"type":"function","title":"OpenAI.Responses.Schema.build_output/1","doc":"Builds a structured output schema from a simple Elixir map or keyword list format.\n\nThe input should be a map or keyword list where:\n- Keys are field names (atoms)\n- Values are either:\n  - A single atom like `:string`, `:number`, `:boolean`, etc.\n  - A tuple like `{:string, description: \"...\", pattern: \"...\"}`\n  - For arrays: `{:array, :string}` or `{:array, %{field: :type}}`\n\nWhen using keyword lists, the order of fields is preserved in the required array.\nWhen using maps, fields are sorted alphabetically in the required array.","ref":"OpenAI.Responses.Schema.html#build_output/1"},{"type":"module","title":"OpenAI.Responses.Stream","doc":"Streaming functionality for the Responses library.\n\nThis module provides functions for streaming responses from the OpenAI API,\nallowing you to process data as it arrives rather than waiting for the complete response.","ref":"OpenAI.Responses.Stream.html"},{"type":"module","title":"Stream Processing Helpers - OpenAI.Responses.Stream","doc":"For processing streamed data, use these helpers:\n\n  * `text_deltas/1` - Extracts text content from event streams\n  * `json_events/1` - Extracts JSON parsing events from response streams","ref":"OpenAI.Responses.Stream.html#module-stream-processing-helpers"},{"type":"module","title":"Examples - OpenAI.Responses.Stream","doc":"# Extract text from streaming response\n    text = Responses.stream(input: \"Write a story\")\n           |> Responses.Stream.text_deltas()\n           |> Enum.join()\n\n    # Process JSON events directly from response stream\n    Responses.stream(input: \"Generate data\", schema: %{name: :string})\n    |> Responses.Stream.json_events()\n    |> Stream.each(&IO.inspect/1)\n    |> Stream.run()","ref":"OpenAI.Responses.Stream.html#module-examples"},{"type":"function","title":"OpenAI.Responses.Stream.delta/1","doc":"Calls the provided function on all the text chunks received from the server.\n\nThis helper handles the wrapped results and only processes successful text delta events.\nErrors and other events are silently ignored.","ref":"OpenAI.Responses.Stream.html#delta/1"},{"type":"function","title":"Examples - OpenAI.Responses.Stream.delta/1","doc":"Responses.create(\n      input: \"Write a story\",\n      stream: Responses.Stream.delta(&IO.write/1)\n    )","ref":"OpenAI.Responses.Stream.html#delta/1-examples"},{"type":"function","title":"OpenAI.Responses.Stream.json_events/1","doc":"Extracts JSON parsing events from a response stream.\n\nThis helper automatically extracts text deltas from the event stream and\nconverts them into JSON parsing events using incremental parsing.","ref":"OpenAI.Responses.Stream.html#json_events/1"},{"type":"function","title":"Events - OpenAI.Responses.Stream.json_events/1","doc":"Returns a Stream that yields JSON parsing events:\n\n  * `:start_object` - Beginning of a JSON object\n  * `:end_object` - End of a JSON object\n  * `:start_array` - Beginning of a JSON array\n  * `:end_array` - End of a JSON array\n  * `{:string, value}` - A string value\n  * `{:integer, value}` - An integer value\n  * `{:float, value}` - A float value\n  * `{:boolean, value}` - A boolean value\n  * `:null` - A null value\n  * `:colon` - Colon between key and value\n  * `:comma` - Comma between elements","ref":"OpenAI.Responses.Stream.html#json_events/1-events"},{"type":"function","title":"Examples - OpenAI.Responses.Stream.json_events/1","doc":"# Process JSON events directly from API response\n    events = Responses.stream(\n      input: \"Give me 10 U.S. presidents\",\n      schema: %{presidents: {:array, %{name: :string, birth_year: :integer}}}\n    )\n    |> Responses.Stream.json_events()\n    |> Enum.into([])\n\n    # Process events as they arrive\n    Responses.stream(\n      input: \"Generate product catalog\",\n      schema: %{products: {:array, %{name: :string, price: :number}}}\n    )\n    |> Responses.Stream.json_events()\n    |> Stream.each(fn event ->\n      IO.inspect(event, label: \"JSON Event\")\n    end)\n    |> Stream.run()","ref":"OpenAI.Responses.Stream.html#json_events/1-examples"},{"type":"function","title":"OpenAI.Responses.Stream.stream/1","doc":"Returns a Stream that yields chunks from the OpenAI API.\n\nThis function returns an Enumerable that yields results wrapped in tuples:\n`{:ok, chunk}` for successful chunks or `{:error, reason}` for parsing errors.","ref":"OpenAI.Responses.Stream.html#stream/1"},{"type":"function","title":"Examples - OpenAI.Responses.Stream.stream/1","doc":"# Process all results with error handling\n    Responses.stream(input: \"Hello\")\n    |> Enum.each(fn\n      {:ok, chunk} -> IO.inspect(chunk)\n      {:error, reason} -> IO.puts(\"Error: #{inspect(reason)}\")\n    end)\n\n    # Get only text deltas, ignoring errors\n    text = Responses.stream(input: \"Write a story\")\n           |> Stream.filter(fn\n             {:ok, %{event: \"response.output_text.delta\"}} -> true\n             _ -> false\n           end)\n           |> Stream.map(fn {:ok, chunk} -> chunk.data[\"delta\"] end)\n           |> Enum.join()","ref":"OpenAI.Responses.Stream.html#stream/1-examples"},{"type":"function","title":"OpenAI.Responses.Stream.stream_with_callback/2","doc":"Stream a response from the OpenAI API with a callback function.\n\nTakes a `callback` function that will be called for each parsed chunk of the stream.\nThe callback receives results wrapped in tuples:\n\n    {:ok, %{event: \"event_type\", data: %{...}}}\n    # or\n    {:error, reason}\n\nThe callback should return `:ok` to continue streaming or `{:error, reason}` to stop.","ref":"OpenAI.Responses.Stream.html#stream_with_callback/2"},{"type":"function","title":"Options - OpenAI.Responses.Stream.stream_with_callback/2","doc":"Accepts the same options as `Responses.create/1`, and automatically adds `stream: true` parameter.","ref":"OpenAI.Responses.Stream.html#stream_with_callback/2-options"},{"type":"function","title":"Examples - OpenAI.Responses.Stream.stream_with_callback/2","doc":"# Simple debugging - print all results\n    Responses.Stream.stream_with_callback(&IO.inspect/1, input: \"Hello\")\n\n    # Process only text deltas using the delta/1 helper\n    Responses.Stream.stream_with_callback(\n      Responses.Stream.delta(&IO.write/1),\n      input: \"Write a story\"\n    )\n\n    # Custom processing with error handling\n    Responses.Stream.stream_with_callback(fn\n      {:ok, %{event: \"response.output_text.delta\", data: %{\"delta\" => text}}} ->\n        IO.write(text)\n        :ok\n      {:ok, %{event: \"response.completed\"}} ->\n        IO.puts(\"\nStream complete!\")\n        :ok\n      {:error, reason} ->\n        IO.puts(\"\nStream error: #{inspect(reason)}\")\n        :ok  # Continue despite error\n      _ ->\n        :ok\n    end, input: \"Tell me a joke\")","ref":"OpenAI.Responses.Stream.html#stream_with_callback/2-examples"},{"type":"function","title":"OpenAI.Responses.Stream.text_deltas/1","doc":"Extracts text deltas from a stream, ignoring errors and other event types.\n\nThis helper transforms a raw event stream into a text-only stream by filtering\nfor `response.output_text.delta` events and extracting their delta content.\nAll errors and non-text events are silently ignored.","ref":"OpenAI.Responses.Stream.html#text_deltas/1"},{"type":"function","title":"Examples - OpenAI.Responses.Stream.text_deltas/1","doc":"# Get text stream from API response\n    text = Responses.stream(input: \"Write a story\")\n           |> Responses.Stream.text_deltas()\n           |> Enum.join()\n\n    # Process text incrementally\n    Responses.stream(input: \"Count to 10\")\n    |> Responses.Stream.text_deltas()\n    |> Stream.each(&IO.write/1)\n    |> Stream.run()","ref":"OpenAI.Responses.Stream.html#text_deltas/1-examples"},{"type":"exception","title":"OpenAI.Responses.Error","doc":"Exception module for OpenAI API errors.\n\nThis module represents errors that can occur when making requests to the OpenAI API.\nIt includes standard OpenAI error fields as well as HTTP status codes.","ref":"OpenAI.Responses.Error.html"},{"type":"exception","title":"Fields - OpenAI.Responses.Error","doc":"* `:message` - Human-readable error description\n* `:code` - OpenAI error code (e.g., \"invalid_api_key\")\n* `:param` - The parameter that caused the error (if applicable)\n* `:type` - OpenAI error type (e.g., \"invalid_request_error\")\n* `:status` - HTTP status code from the response","ref":"OpenAI.Responses.Error.html#module-fields"},{"type":"exception","title":"Examples - OpenAI.Responses.Error","doc":"iex> error = %OpenAI.Responses.Error{\n    ...>   message: \"Rate limit exceeded\",\n    ...>   code: \"rate_limit_exceeded\",\n    ...>   type: \"rate_limit_exceeded\",\n    ...>   status: 429\n    ...> }\n    iex> OpenAI.Responses.Error.retryable?(error)\n    true","ref":"OpenAI.Responses.Error.html#module-examples"},{"type":"function","title":"OpenAI.Responses.Error.from_response/1","doc":"","ref":"OpenAI.Responses.Error.html#from_response/1"},{"type":"function","title":"OpenAI.Responses.Error.retryable?/0","doc":"","ref":"OpenAI.Responses.Error.html#retryable?/0"},{"type":"function","title":"OpenAI.Responses.Error.retryable?/1","doc":"Determines if an error is retryable.\n\nReturns `true` if the error is due to temporary server issues that may resolve\nwith retry attempts, such as rate limits, internal server errors, or network\ntimeouts. Returns `false` for client errors like malformed requests that will\nnot succeed on retry.","ref":"OpenAI.Responses.Error.html#retryable?/1"},{"type":"function","title":"Retryable conditions - OpenAI.Responses.Error.retryable?/1","doc":"","ref":"OpenAI.Responses.Error.html#retryable?/1-retryable-conditions"},{"type":"function","title":"HTTP status codes - OpenAI.Responses.Error.retryable?/1","doc":"* `429` - Rate limit or quota exceeded\n* `500` - Internal server error\n* `503` - Service overloaded / Slow down","ref":"OpenAI.Responses.Error.html#retryable?/1-http-status-codes"},{"type":"function","title":"Transport errors - OpenAI.Responses.Error.retryable?/1","doc":"* `:timeout` - Request timeout\n* `:closed` - Connection closed","ref":"OpenAI.Responses.Error.html#retryable?/1-transport-errors"},{"type":"function","title":"Examples - OpenAI.Responses.Error.retryable?/1","doc":"iex> error = %OpenAI.Responses.Error{status: 429}\n    iex> OpenAI.Responses.Error.retryable?(error)\n    true\n\n    iex> error = %OpenAI.Responses.Error{status: 400}\n    iex> OpenAI.Responses.Error.retryable?(error)\n    false\n\n    iex> error = %Req.TransportError{reason: :timeout}\n    iex> OpenAI.Responses.Error.retryable?(error)\n    true","ref":"OpenAI.Responses.Error.html#retryable?/1-examples"},{"type":"extras","title":"OpenAI.Responses","doc":"# OpenAI.Responses\n\nA client library for the OpenAI Responses API with automatic text extraction and cost calculation.","ref":"readme.html"},{"type":"extras","title":"Installation - OpenAI.Responses","doc":"Add `openai_responses` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:openai_responses, \"~> 0.5.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - OpenAI.Responses","doc":"Set your OpenAI API key using one of these methods:","ref":"readme.html#configuration"},{"type":"extras","title":"Environment Variable - OpenAI.Responses","doc":"```bash\nexport OPENAI_API_KEY=\"your-api-key\"\n```","ref":"readme.html#environment-variable"},{"type":"extras","title":"Application Config - OpenAI.Responses","doc":"```elixir\nconfig :openai_responses, :openai_api_key, \"your-api-key\"\n```","ref":"readme.html#application-config"},{"type":"extras","title":"Getting Started - OpenAI.Responses","doc":"For a comprehensive tutorial and examples, see the [interactive tutorial](tutorial.livemd) in Livebook.","ref":"readme.html#getting-started"},{"type":"extras","title":"Advanced Examples - OpenAI.Responses","doc":"","ref":"readme.html#advanced-examples"},{"type":"extras","title":"Simple terminal chat - OpenAI.Responses","doc":"```elixir\ndefmodule Chat do\n  alias OpenAI.Responses\n\n  def run do\n    IO.puts(\"Simple AI Chat (type /exit or /quit to end)\")\n    IO.puts(\"=\" |> String.duplicate(40))\n\n    loop(nil)\n  end\n\n  defp loop(previous_response) do\n    input = IO.gets(\"\\nYou: \") |> String.trim()\n\n    case input do\n      cmd when cmd in [\"/exit\", \"/quit\"] ->\n        IO.puts(\"\\nGoodbye!\")\n\n      _ ->\n        IO.write(\"\\nAI: \")\n\n        # Use previous response for context, or create new conversation\n        response = if previous_response do\n          # Continue conversation with context\n          Responses.create!(\n            previous_response,\n            input: input,\n            stream: Responses.Stream.delta(&IO.write/1)\n          )\n        else\n            # First message - start new conversation\n            Responses.create!(\n              input: input,\n              stream: Responses.Stream.delta(&IO.write/1)\n            )\n        end\n\n        IO.puts(\"\")  # Add newline after response\n        loop(response)\n    end\n  end\nend\n\n# Run the chat\nChat.run()\n```","ref":"readme.html#simple-terminal-chat"},{"type":"extras","title":"Streaming with Structured Output - OpenAI.Responses","doc":"```elixir\n# Stream a JSON response with structured output\nResponses.stream(\n  input: \"List 3 programming languages with their year of creation\",\n  model: \"gpt-4o-mini\",\n  schema: %{\n    languages: {:array, %{\n      name: :string,\n      year: :integer,\n      paradigm: {:string, description: \"Main programming paradigm\"}\n    }}\n  }\n)\n|> Responses.Stream.json_events()\n|> Enum.each(&IO.puts/1)\n```","ref":"readme.html#streaming-with-structured-output"},{"type":"extras","title":"Cost Tracking with High Precision - OpenAI.Responses","doc":"```elixir\n{:ok, response} = Responses.create(\"Explain quantum computing\")\n\n# All cost values are Decimal for precision\nIO.inspect(response.cost)\n# => %{\n#      input_cost: #Decimal<0.0004>,\n#      output_cost: #Decimal<0.0008>,\n#      total_cost: #Decimal<0.0012>,\n#      cached_discount: #Decimal<0>\n#    }\n\n# Convert to float if needed\ntotal_in_cents = response.cost.total_cost |> Decimal.mult(100) |> Decimal.to_float()\n```","ref":"readme.html#cost-tracking-with-high-precision"},{"type":"extras","title":"Documentation - OpenAI.Responses","doc":"- [API Documentation](https://hexdocs.pm/openai_responses)\n- [Interactive Tutorial](tutorial.livemd)\n- [GitHub Repository](https://github.com/vkryukov/openai-responses)","ref":"readme.html#documentation"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"0.5.1 - Changelog","doc":"","ref":"changelog.html#0-5-1"},{"type":"extras","title":"Added - Changelog","doc":"- **Map support for options** - Multiple functions now accept maps in addition to keyword lists for options, providing more flexibility and consistency with Elixir conventions:\n  - `create/1` and `create/2` - Create responses with map or keyword list options\n  - `stream/1` - Stream responses using map or keyword list options\n  - `run/2` - Run function calling conversations with map or keyword list options\n  - `OpenAI.Responses.Stream.stream/1` and `OpenAI.Responses.Stream.stream_with_callback/2` - Stream functions also accept maps\n  - Maps with mixed atom and string keys are supported\n  - Example: `Responses.create(%{input: \"Hello\", model: \"gpt-4o\"})`","ref":"changelog.html#added"},{"type":"extras","title":"0.5.0 - Changelog","doc":"","ref":"changelog.html#0-5-0"},{"type":"extras","title":"Added - Changelog","doc":"- **Union type support via `anyOf`** - `Schema.build_output/1` and `Schema.build_function/3` now support union types through the `anyOf` specification. This allows defining properties that can be one of multiple types:\n  - `{:anyOf, [:string, :number]}` - tuple syntax for simple unions\n  - `[:anyOf, [:string, :number]]` - list syntax for simple unions\n  - Complex unions with objects and nested structures are fully supported\n- **LLM usage guide** - Added comprehensive `usage-rules.md` documentation with detailed examples and best practices for using the OpenAI.Responses library with LLM agents\n- **Manual function calling with `call_functions/2`** - Added public function to execute function calls from a response and format results for the API. This enables custom workflows where users need to intercept, modify, or add context to function results before continuing the conversation. Function return values are passed through without string conversion, so they must be JSON-encodable (maps, lists, strings, numbers, booleans, nil)\n- **Error module with retry support** - Added `OpenAI.Responses.Error` exception module that:\n  - Represents OpenAI API errors with fields for message, code, param, type, and HTTP status\n  - Provides `retryable?/1` function to determine if errors can be retried\n  - Supports both HTTP status codes (429, 500, 503) and Req transport errors (timeout, closed)","ref":"changelog.html#added-1"},{"type":"extras","title":"Fixed - Changelog","doc":"- **Model preservation in follow-up responses** - `create/2` now correctly preserves the model from the previous response when creating follow-ups, instead of defaulting to the standard model. The model can still be explicitly overridden when needed","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- **Package metadata** - Updated `mix.exs` to include additional files in the hex package distribution","ref":"changelog.html#changed"},{"type":"extras","title":"0.4.2 - Changelog","doc":"","ref":"changelog.html#0-4-2"},{"type":"extras","title":"Added - Changelog","doc":"- **List format support for Schema definitions** - `Schema.build_output/1` and `Schema.build_function/3` now accept list syntax `[type, options]` in addition to tuple syntax `{type, options}`. This provides more flexibility when defining schemas:\n  - `[:string, %{description: \"Full name\"}]` - list with map options\n  - `[:string, [description: \"Full name\"]]` - list with keyword list options\n  - `[:array, :string]` - list format for arrays\n  - All formats can be mixed within the same schema definition\n- Pricing for o3 pro model","ref":"changelog.html#added-2"},{"type":"extras","title":"Changed - Changelog","doc":"- **Cost calculation for unknown models** - When pricing data is not available for a model, `Response.calculate_cost/1` now returns zero costs for all categories instead of `nil`. This provides a consistent cost structure regardless of whether the model has pricing information.\n- **Schema internals refactored** - The schema building logic now uses a two-phase approach (normalize then build) making it more maintainable and extensible. This is an internal change that doesn't affect the public API.\n- Pricing for o3 model updated to reflect 80% price decrease","ref":"changelog.html#changed-1"},{"type":"extras","title":"0.4.1 - Changelog","doc":"","ref":"changelog.html#0-4-1"},{"type":"extras","title":"Added - Changelog","doc":"- **Support for string tuple syntax in Schema arrays** - `Schema.build_property/1` now accepts `{\"array\", item_type}` in addition to `{:array, item_type}` for defining array types in schemas\n- **Support for string keys in Schema definitions** - `Schema.build_output/1` and `Schema.build_function/3` now accept lists with string keys in addition to keyword lists with atom keys. This enables schemas to be stored in databases or other persistence layers where atoms are converted to strings, and then reused without conversion back to atoms","ref":"changelog.html#added-3"},{"type":"extras","title":"0.4.0 - Changelog","doc":"Major refactoring to make the codebase more modular and maintainable, and simplify the API.","ref":"changelog.html#0-4-0"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- **Removed `:client` option** from `create/1`\n- **Removed `Responses.Helpers` module** - cost is now calculated automatically for models with known pricing\n- **Removed image helpers** for now until a better API can be designed\n- **Removed `delete/2`, `get/2`, and `list_input_items/2`** - use the low-level `request/1` function instead\n- **Removed `parse/2`** - use `create/1` with a `:schema` option instead\n- **Removed `collect_stream/1`**\n- **Changed schema syntax** - replaced `Schema.object/2`, `Schema.string/1` etc. with simple maps","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Added - Changelog","doc":"- `create/2` can be used to follow up on a previous response\n- `create!/1` and `create!/2` raise an error if the response is not successful\n- `run/2` and `run!/2` functions for automatic function calling loops\n  - Takes a list of available functions and automatically handles function calls\n  - Continues conversation until a final response without function calls is received\n  - Returns a list of all responses generated during the conversation\n  - Supports both map and keyword list function definitions\n- `list_models/0` and `list_models/1` to return the list of available OpenAI models\n- `:schema` option to `create/1` to specify the schema of the response body\n- `Schema.build_output/1` to build the output schema for the response body\n- `Schema.build_function/3` to build the function schema for tool calling\n- `:stream` callback function option to `create/1` that will be called on every event\n- `json_events/1` helper to `Responses.Stream` for streaming JSON events with Jaxon","ref":"changelog.html#added-4"},{"type":"extras","title":"Changed - Changelog","doc":"- `create/1` now returns a `Response` struct containing the generated response body, text, parsed JSON, and cost information\n- `text_deltas/1` is moved to `Responses.Stream`","ref":"changelog.html#changed-2"},{"type":"extras","title":"0.3.0 - Changelog","doc":"","ref":"changelog.html#0-3-0"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- **Removed positional parameters** from `create`, `stream`, and `parse` functions. Use keyword lists or maps for parameters instead.\n\nInstead of `OpenAIResponses.create(\"model\", \"prompt\")`, use `OpenAIResponses.create(model: \"model\", prompt: \"prompt\")`.\n\n- **Changed parse response format**. The structure of `parse` return has changed: instead of the parsed response,\n  it now returns a map with the following keys:\n  - parsed: the parsed result\n  - raw_response: the raw response received from OpenAI, which can be used to e.g. calculate the cost\n  - token_usage: information about the token usage\n\n- **Removed Config module**. Configuration handling has been streamlined. (Commit: 88b812e)\n- **Removed parse_stream function**. This function was not working as intended, and is removed in this release. (Commit: 9723ccb)\n- **Removed explicit instructions message in parse**. This should now be handled by the developer. (Commit: f8addc0)","ref":"changelog.html#breaking-changes-1"},{"type":"extras","title":"Added - Changelog","doc":"- **Cost calculations** for various models including 4.1 models and chatgpt-4o-latest. (Commits: 2985d94, 70d68ad, 4f2cdc3)\n- **Option to log requests** for debugging or monitoring purposes. (Commit: d4d56b5)","ref":"changelog.html#added-5"},{"type":"extras","title":"Changed - Changelog","doc":"- **Increased default timeout** to 60 seconds from a previous value. (Commit: 4a01b66)","ref":"changelog.html#changed-3"},{"type":"extras","title":"Fixed - Changelog","doc":"- **Fixed tests** across multiple commits to ensure functionality and improve coverage. (Commits: c78858c, 18c535a, 804522e, c594688, d637a73)","ref":"changelog.html#fixed-1"},{"type":"extras","title":"Documentation - Changelog","doc":"- **Updated documentation** to reflect new features, response usage, and tutorial content. (Commits: d1bc204, f07d990, 03031e6, a4ac509)","ref":"changelog.html#documentation"},{"type":"extras","title":"Tutorial","doc":"# Tutorial\n\n```elixir\nMix.install([\n  {:openai_responses, path: \"~/src/responses\"},\n  {:kino, \"~> 0.11.0\"}\n])\n```","ref":"tutorial.html"},{"type":"extras","title":"Introduction - Tutorial","doc":"OpenAI.Responses is a [Small Development Kit](https://dashbit.co/blog/sdks-with-req-stripe) for the OpenAI [Responses](https://platform.openai.com/docs/api-reference/responses) API. It can get you started in no time, and automatically supports conversation state, structured responses, streaming, function calls, and cost calculations.\n\nIf you want an industrial-grade library that supports multiple providers, you should instead consider one of [LangChain](https://hexdocs.pm/langchain/readme.html), [OpenaiEx](https://hexdocs.pm/openai_ex/OpenaiEx.html), or [Instructor](https://hexdocs.pm/instructor/Instructor.html). If, however, you want to start as quickly as possible without learning a new framework first, and are ready to trade the flexibility of choosing an LLM provider for the conveniences of OpenAI's latest API, you've come to the right place!\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#introduction"},{"type":"extras","title":"Basic usage - Tutorial","doc":"The basic usage is simple: set the `OPENAI_API_KEY` environment variable and add `openai_responses` to your `mix.exs`\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\ndef deps do\n  [\n    # ...\n    {:openai_responses, \"~> 0.5.0\"}\n  ]\nend\n```\n\nis enough to get you started:\n\n```elixir\nalias OpenAI.Responses\n\n{:ok, response} = Responses.create(\"Write me a haiku about Elixir\")\n# -> {:ok, %OpenAI.Responses.Response{text: ..., ...}}\n\nIO.puts(response.text)\n```\n\n`create(s) when is_binary(s)` is a shortcut for `create(input: s)`. In general, `create/1` takes a keyword list with anything that [Create a model response](https://platform.openai.com/docs/api-reference/responses/create) supports. `create!/1` is a version of `create/1` that raises on errors:\n\n```elixir\nresponse =\n  Responses.create!(\n    input: [\n      %{role: :developer, content: \"Talk like a pirate.\"},\n      %{role: :user, content: \"Write me a haiku about Elixir\"}\n    ],\n    model: \"o4-mini\",\n    reasoning: %{effort: :low}\n  )\n\nIO.puts(\"#{response.text}\\n\\nCost: $#{response.cost.total_cost}\")\n```\n\nThere is also `create/2` and `create!/2` that take `%Responses.Response{}` as a first argument and keyword list as a second to automatically handle the [conversation state](https://platform.openai.com/docs/guides/conversation-state?api-mode=responses):\n\n```elixir\nResponses.create!(\n  input: [\n    %{role: :developer, content: \"Talk like a pirate.\"},\n    %{role: :user, content: \"Write me a haiku about Elixir\"}\n  ]\n)\n|> Responses.create!(input: \"Which programming language is this haiku about?\")\n|> Map.get(:text)\n|> IO.puts()\n```\n\nNotice that the follow-up response also talks like a pirate, as it inherits the model, developer settings, etc. from the initial response.\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#basic-usage"},{"type":"extras","title":"Structured Output - Tutorial","doc":"One of the coolest features of the latest generation of OpenAI models is their ability to output [Structured Output](https://platform.openai.com/docs/guides/structured-outputs?api-mode=responses) that **precisely matches** the supplied JSON schema. `create/1` accepts a `schema:` option that allows easy creation of such schema in the required format.\n\nThe return result will automatically parse the response into `%Response{parsed: object}`.\n\n```elixir\nalias OpenAI.Responses\n\nResponses.create!(\n  input: \"List facts about first 3 U.S. Presidents\",\n  schema: %{\n    presidents:\n      {:array,\n       %{\n         name: :string,\n         birth_year: :integer,\n         little_known_facts: {:array, {:string, max_items: 2}}\n       }}\n  }\n)\n|> Map.get(:parsed)\n|> Map.get(\"presidents\")\n```\n\nThe above example should give you enough to get started, and you can read the documentation about [supported schemas](https://platform.openai.com/docs/guides/structured-outputs#supported-schemas).\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#structured-output"},{"type":"extras","title":"Streaming - Tutorial","doc":"Streaming model responses is a great way to increase interactivity of your application. There are two ways to support streaming with `OpenAI.Responses`:\n\n1. By adding a `callback: callback_fn` option to `create/1`. Here `callback_fn/1` takes a map `%{event: type, data: data}` and should return either `:ok` to continue streaming or `{:error, reason}` to stop.\n\n   The call will be blocked until the streaming ends but will otherwise return the same\n   `%Response{}` structure.\n\n2. By calling `stream/1`, which returns an Elixir Stream of `%{event: type, data: data}` objects.\n\nFor the supported event types and data format, refer to the [Streaming Responses API](https://platform.openai.com/docs/api-reference/responses-streaming/response) docs.\n\nHere is an example of how this works. We use the `Responses.Stream.text_deltas` helper to transform the stream of events into a stream of text chunks, and use `Kino.Frame` to demonstrate interactive output updates in a Livebook.\n\n```elixir\nalias OpenAI.Responses\n\nframe = Kino.Frame.new()\nKino.render(frame)\n\nResponses.stream(\n  input: \"\"\"\n  Write a short fairy tale about an Elixir developer who tried to use Java,\n  and about the horrors that have ensued.\n  \"\"\",\n  temperature: 0.7\n)\n|> Responses.Stream.text_deltas()\n|> Stream.each(fn delta ->\n  Kino.Frame.append(frame, Kino.Markdown.new(delta, chunk: true))\nend)\n|> Stream.run()\n\n:done\n```\n\nThere is also a `Responses.Stream.json_events/1` helper, which uses the `Jaxon` library to stream JSON events:\n\n```elixir\nResponses.stream(\n  input: \"Tell me about first 2 U.S. Presidents\",\n  schema: %{presidents: {:array, %{name: :string, birth_year: :integer}}}\n)\n|> Responses.Stream.json_events()\n|> Stream.each(&IO.inspect/1)\n|> Stream.run()\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"tutorial.html#streaming"},{"type":"extras","title":"Tools - Tutorial","doc":"Using OpenAI built-in tools, for example [Web Search](https://platform.openai.com/docs/guides/tools-web-search?api-mode=responses), is simple: just add a `tools:` parameter to `create/1`:\n\n```elixir\nalias OpenAI.Responses\n\nResponses.create!(\n  input: \"Summarize in 3 paragraphs a positive news story from today\",\n  tools: [%{type: \"web_search_preview\"}]\n)\n|> Map.get(:text)\n|> IO.puts\n```\n\nFor [Function calling](https://platform.openai.com/docs/guides/function-calling?api-mode=responses), you can provide a function description. The `Responses.Schema.build_function/3` helper makes this easier:\n\n```elixir\n# Using the build_function helper\nweather_tool = Responses.Schema.build_function(\n  \"get_weather\",\n  \"Get current temperature for a given location\",\n  %{location: {:string, description: \"City and country e.g. Bogotá, Colombia\"}}\n)\n\nresponse = Responses.create!(\n  input: \"What is the weather like in Paris today?\",\n  tools: [weather_tool]\n)\n\nresponse.function_calls\n```\n\nThe resulting `%Response{}` has a `:function_calls` field populated with the function calls requested by the model. Our app can now call each function, and we can provide results back to the model:\n\n```elixir\nresponse\n|> Responses.create!(\n  input: [%{\n    type: \"function_call_output\",\n    call_id: response.function_calls |> List.first() |> Map.get(:call_id),\n    output: \"15C\"\n  }]\n)\n|> Map.get(:text)\n```","ref":"tutorial.html#tools"},{"type":"extras","title":"Automating Function Calls with run/2 - Tutorial","doc":"The `Responses.run/2` function automates the process of handling function calls. It will repeatedly call your functions and feed the results back to the model until a final response is achieved:\n\n```elixir\n# Define available functions\nfunctions = %{\n  \"get_weather\" => fn %{\"location\" => location} ->\n    # In a real app, this would call a weather API\n    case location do\n      \"Paris\" -> \"15°C, partly cloudy\"\n      \"London\" -> \"12°C, rainy\"\n      \"New York\" -> \"8°C, sunny\"\n      _ -> \"Weather data not available\"\n    end\n  end,\n  \"get_time\" => fn %{\"timezone\" => timezone} ->\n    # In a real app, this would get actual time for timezone\n    case timezone do\n      \"Europe/Paris\" -> \"14:30\"\n      \"Europe/London\" -> \"13:30\"\n      \"America/New_York\" -> \"08:30\"\n      _ -> \"Unknown timezone\"\n    end\n  end\n}\n\n# Define function tools\nweather_tool = Responses.Schema.build_function(\n  \"get_weather\",\n  \"Get current weather for a location\",\n  %{location: {:string, description: \"City name\"}}\n)\n\ntime_tool = Responses.Schema.build_function(\n  \"get_time\",\n  \"Get current time in a timezone\",\n  %{timezone: {:string, description: \"Timezone like Europe/Paris\"}}\n)\n\n# Run the conversation with automatic function calling\nresponses = Responses.run(\n  [\n    input: \"What's the weather and time in Paris?\",\n    tools: [weather_tool, time_tool]\n  ],\n  functions\n)\n\n# The last response contains the final answer\nresponses |> List.last() |> Map.get(:text) |> IO.puts()\n\n# You can also inspect all intermediate responses\nIO.puts(\"\\nTotal responses: #{length(responses)}\")\n```\n\nThe `run/2` function returns a list of all responses generated during the conversation. This allows you to:\n\n* Track the conversation flow\n* See what functions were called\n* Calculate total costs across all API calls\n* Debug issues in function calling\n\nThere's also a `run!/2` variant that raises on errors instead of returning error tuples.","ref":"tutorial.html#automating-function-calls-with-run-2"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}